% BMP229, Bloch Equation Simulation
% Balanced SSFP frequency response
%
% Modified: 2014/04/14
% Created: 2014/03/17
% Holden Wu

%close all; clear all;

% Tissue params
VARY_T2 = 1;
if( VARY_T2 )
  % Fix T1, vary T2
  T1vec = [1000]; %ms
  T2vec = [100, 200, 500, 1000]; %ms
else
  % Fix T2, vary T1
  T1vec = [100, 200, 500, 1000]; %ms
  T2vec = [40]; %ms
end

% Sequence sim params
% note: a coarser grid is used for SSFP-FID/Echo to reduce run time
MODE = 2;
if( 1==MODE )
  % bSSFP sequence params
  TRms = 5; TEms = TRms/2;
  % flip angle
  RFthdeg = [0:1:180];
  RFset = [11 51 91]; % for now, hard-coded indices to select RFth = 10, 50, 90 deg
  % off-resonance phase accrual
  dfvec = [-400:20:400]; %Hz
elseif( 2==MODE )
  % SSFP-FID
  TRms = 10; TEms = 2; FISP = 1;
  % flip angle
  RFthdeg = [0:10:180];
  RFset = [2 6 10];
  % off-resonance phase accrual
  dfvec = [-400:50:400]; %Hz
else    
  % SSFP-Echo
  TRms = 10; TEms = 8; FISP = 0;
  % flip angle
  RFthdeg = [0:10:180];
  RFset = [2 6 10];
  % off-resonance phase accrual
  dfvec = [-400:50:400]; %Hz
end

% flip angle
%RFthdeg = [0:1:180];
RFthrad = RFthdeg * pi/180;
RFthplot = 60;
[y, RFI] = min(abs(RFthdeg-RFthplot));
% off-resonance phase accrual
%dfvec = [-400:20:400]; %Hz
dfplot = 1000/TRms/2;
[y, dfI] = min(abs(dfvec-dfplot));

Mxyss = zeros(numel(T1vec), numel(T2vec), numel(RFthdeg), numel(dfvec));
for t1idx=1:numel(T1vec),
  T1ms = T1vec(t1idx);
  for t2idx=1:numel(T2vec),
    T2ms = T2vec(t2idx);
    %-------------------------------------------------------------------------
    % Signal as a function of flip angle
    Mss = zeros(3, numel(RFthrad));
    %beta = betarad;
    for rfidx = 1:numel(RFthrad),
      RFth = RFthrad(rfidx);
      for dfidx = 1:numel(dfvec),
        df = dfvec(dfidx);
        if( 1==MODE )
          [Msig0, Mss0] = sssignal(RFth,T1ms,T2ms,TEms,TRms,df);
        else  
          [Msig0, Mss0] = gresignal(RFth,T1ms,T2ms,TEms,TRms,df, FISP);
        end  
        Mxyss(t1idx,t2idx,rfidx,dfidx) = Msig0;
      end  % df loop  
    end % FA loop
    %-------------------------------------------------------------------------
  end % T2 loop
end % T1 loop

figure(101);
% magnitude
subplot(2,1,1);
myplot=plot(dfvec, squeeze(abs(Mxyss(:,:,RFI,:)))); 
if( 1==numel(T1vec) )
  mylgnd=legend(num2str(T2vec.'/T1ms));
else
  mylgnd=legend(num2str(T2ms./T1vec.'));
end  
xlabel('off-resonance (Hz)'); ylabel('Mxy,ss (normalized mag)');
title(sprintf('Different T2/T1 ratios, \\theta = %d deg', RFthdeg(RFI)));
xlim([min(dfvec), max(dfvec)]);
% modify plot appearance
set(gcf,'Color','k');
set(gca,'Color','k','XColor','w','YColor','w');
set(myplot,'LineWidth',1.5);
set(mylgnd,'color','none','TextColor','w');
set(get(gca,'Title'),'Color','w','FontSize',16);
set(get(gca,'Xlabel'),'FontSize',16);
set(get(gca,'Ylabel'),'FontSize',16);
% phase
subplot(2,1,2);
myplot2=plot(dfvec, squeeze(angle(Mxyss(:,:,RFI,:))));
if( 1==numel(T1vec) )
  mylgnd=legend(num2str(T2vec.'/T1ms));
else
  mylgnd=legend(num2str(T2ms./T1vec.'));
end  
xlabel('off-resonance (Hz)'); ylabel('Mxy,ss (phase)');
xlim([min(dfvec), max(dfvec)]);
% modify plot appearance
set(gcf,'Color','k');
set(gca,'Color','k','XColor','w','YColor','w');
set(myplot2,'LineWidth',1.5);
set(mylgnd,'color','none','TextColor','w');
set(get(gca,'Title'),'Color','w','FontSize',16);
set(get(gca,'Xlabel'),'FontSize',16);
set(get(gca,'Ylabel'),'FontSize',16);

if( 1==numel(T1vec) )
  profiles = squeeze( Mxyss(1,end,RFset,:) ).';
else
  profiles = squeeze( Mxyss(end,1,RFset,:) ).';
end  
figure(102);
myplot=plot(dfvec, abs(profiles)); mylgnd=legend(num2str(RFthdeg(RFset).'));
xlabel('off-resonance (Hz)'); ylabel('Mxy,ss (normalized mag)');
title(sprintf('T2/T1 = %.2f, different thetas', T2vec(end)/T1vec(end)));
xlim([min(dfvec), max(dfvec)]);
% modify plot appearance
set(gcf,'Color','k');
set(gca,'Color','k','XColor','w','YColor','w');
set(myplot,'LineWidth',1.5);
set(mylgnd,'color','none','TextColor','w');
set(get(gca,'Title'),'Color','w','FontSize',16);
set(get(gca,'Xlabel'),'FontSize',16);
set(get(gca,'Ylabel'),'FontSize',16);

figure(103); 
myplot=plot(RFthdeg, squeeze(abs(Mxyss(:,:,:,dfI))));
if( 1==numel(T1vec) )
  mylgnd=legend(num2str(T2vec.'/T1ms));
else
  mylgnd=legend(num2str(T2ms./T1vec.'));
end  
xlabel('RF flip angle \theta'); ylabel('Mxy,ss (normalized)');
title(sprintf('Different T2/T1 ratios, df = %d Hz', dfvec(dfI)));
xlim([min(RFthdeg), max(RFthdeg)]);
% modify plot appearance
set(gcf,'Color','k');
set(gca,'Color','k','XColor','w','YColor','w');
set(myplot,'LineWidth',1.5);
set(mylgnd,'color','none','TextColor','w');
set(get(gca,'Title'),'Color','w','FontSize',16);
set(get(gca,'Xlabel'),'FontSize',16);
set(get(gca,'Ylabel'),'FontSize',16);

